# Generated by Django 4.2.1 on 2023-07-05 02:16

from django.db import migrations
import django.db.models.manager
import edc_sites.model_mixins
import intecomm_group.models.patient_group
import intecomm_screening.model_mixins.patient_call_model_mixin


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0013_alter_historicalpatientgroup_status_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="communitycarelocation",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Community Care Location",
                "verbose_name_plural": "Community Care Locations",
            },
        ),
        migrations.AlterModelOptions(
            name="patientfollowupcall",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ["report_datetime"],
                "verbose_name": "Patient Followup Call",
                "verbose_name_plural": "Patient Followup Calls",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroup",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Patient Group",
                "verbose_name_plural": "Patient Groups",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroupappointment",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Patient Group Appointment",
                "verbose_name_plural": "Patient Group Appointments",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroupmeeting",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
        migrations.AlterModelManagers(
            name="communitycarelocation",
            managers=[
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="patientfollowupcall",
            managers=[
                (
                    "objects",
                    intecomm_screening.model_mixins.patient_call_model_mixin.Manager(),
                ),
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="patientgroup",
            managers=[
                ("objects", intecomm_group.models.patient_group.PatientGroupManager()),
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 4.2.1 on 2023-07-06 02:18

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_ae", "0003_alter_deathreporttmg_cause_of_death_agreed_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aefollowup",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "AE Follow-up Report",
            },
        ),
        migrations.AlterModelOptions(
            name="aeinitial",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "AE Initial Report",
            },
        ),
        migrations.AlterModelOptions(
            name="aesusar",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "AE SUSAR Report",
            },
        ),
        migrations.AlterModelOptions(
            name="aetmg",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "AE TMG Report",
            },
        ),
        migrations.AlterModelOptions(
            name="deathreport",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Death Report",
            },
        ),
        migrations.AlterModelOptions(
            name="deathreporttmg",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Death Report TMG (1st)",
                "verbose_name_plural": "Death Report TMG (1st)",
            },
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-07 22:50

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0092_healtheconomicsproperty_land_surface_area_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="About how much is this worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_value_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much is this worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="About how much is this worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_value_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much is this worth in total?",
            ),
        ),
    ]



import datetime as dt
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.test import TestCase, override_settings
from edc_constants.constants import HIV, NOT_APPLICABLE, YES
from edc_metadata import REQUIRED
from edc_pdutils import SYSTEM_COLUMNS
from edc_utils import get_utcnow
from model_bakery import baker

from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin
from intecomm_subject.forms import HivReviewForm
from intecomm_subject.models import ClinicalReview

utc_tz = ZoneInfo("UTC")


@time_machine.travel(dt.datetime(2019, 5, 11, 8, 00, tzinfo=utc_tz))
class TestInitialAndReview(IntecommTestCaseMixin, TestCase):
    """These tests cover funcs/methods that access other models such
    as SubjectConsent, SubjectScreening ... .

    More tests are in the `intecomm_form_validators` module.
    """

    patient_group = None

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.patient_group = cls.get_randomized_patient_group(
            report_datetime=get_utcnow() - relativedelta(days=10)
        )

    def setUp(self):
        self.assertTrue(self.patient_group.randomized)

    @override_settings(EDC_MODEL_REPORT_DATETIME_FIELD_NAME="report_datetime")
    def test_ok(self):
        patient_log = self.patient_group.patients.filter(conditions__name__in=[HIV]).first()
        subject_visit = self.get_subject_visit(
            subject_identifier=patient_log.subject_identifier
        )

        self.get_or_create_clinical_review_baseline(patient_log, subject_visit)
        self.assertTrue(
            self.get_crf_metadata(
                subject_visit, model="intecomm_subject.hivinitialreview", entry_status=REQUIRED
            )
        )
        # report rx_init and rx_init_date
        baker.make_recipe(
            "intecomm_subject.hivinitialreview",
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            rx_init=YES,
            rx_init_date=subject_visit.report_datetime - relativedelta(years=5),
        )

        # go to next visit
        subject_visit = self.get_next_subject_visit(subject_visit)
        traveller = time_machine.travel(subject_visit.report_datetime)
        traveller.start()

        obj = baker.prepare_recipe(
            "intecomm_subject.hivreview",
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
        )
        excluded_columns = ["id", "consent_model", "consent_version"]
        excluded_columns.extend(SYSTEM_COLUMNS)
        cleaned_data = {
            k: v
            for k, v in obj.__dict__.items()
            if k not in excluded_columns
            and not k.startswith("_")
            and k not in ["rx_init_estimated_data"]
        }

        # try to submit without clinical review
        cleaned_data.update(
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            rx_init=YES,
            rx_init_date=subject_visit.report_datetime - relativedelta(years=5),
        )
        form = HivReviewForm(data=cleaned_data)

        form.is_valid()

        self.assertIn(ClinicalReview._meta.verbose_name, str(form._errors.get("__all__")))

        # submit clinical review
        baker.make_recipe(
            "intecomm_subject.clinicalreview",
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            hiv_test=NOT_APPLICABLE,
        )

        # incorrectly try to report rx_init date already reported on initial_review
        cleaned_data.update(
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            rx_init=YES,
            rx_init_date=subject_visit.report_datetime - relativedelta(years=5),
        )
        form = HivReviewForm(data=cleaned_data)

        form.is_valid()

        self.assertIn("rx_init", form._errors)

        # ok
        cleaned_data.update(
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            rx_init=NOT_APPLICABLE,
            rx_init_date=None,
        )
        form = HivReviewForm(data=cleaned_data)

        form.is_valid()

        self.assertEqual(form._errors or {}, {})



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_model_fields.widgets import SliderWidget
from intecomm_form_validators.subject import HivMedicationAdherenceFormValidator

from ..models import HivMedicationAdherence


class HivMedicationAdherenceForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HivMedicationAdherenceFormValidator

    visual_score_slider = forms.CharField(
        label="Visual score for your HIV medication",
        widget=SliderWidget(attrs={"min": 0, "max": 100}),
    )

    class Meta:
        model = HivMedicationAdherence
        fields = "__all__"
        labels = {
            "last_missed_pill": (
                "When was the last time you missed taking your HIV medication?"
            ),
            "missed_pill_reason": (
                "If your HIV medication was not taken, select a reason or reasons"
            ),
        }



# Generated by Django 4.1.2 on 2022-11-22 23:57

from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0010_historicalpatientlog_age_in_years_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalpatientlog",
            old_name="hf_identifier",
            new_name="hospital_identifier",
        ),
        migrations.RenameField(
            model_name="patientlog",
            old_name="hf_identifier",
            new_name="hospital_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalpatientlog",
            name="last_4_hf_identifier",
        ),
        migrations.RemoveField(
            model_name="patientlog",
            name="last_4_hf_identifier",
        ),
        migrations.AddField(
            model_name="historicalpatientlog",
            name="last_4_hospital_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                editable=False,
                help_text="auto-populated (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Last 4 digits of hospital_identifier",
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="last_4_hospital_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                editable=False,
                help_text="auto-populated (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Last 4 digits of hospital_identifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_sites.models import SiteModelMixin

from intecomm_screening.model_mixins import PatientCallModelMixin


class PatientFollowupCall(PatientCallModelMixin, SiteModelMixin, BaseUuidModel):
    patient_log = models.ForeignKey("intecomm_screening.PatientLog", on_delete=models.PROTECT)

    def __str__(self):
        return self.patient_log.subject_identifier

    def natural_key(self):
        return (self.patient_log,)

    natural_key.dependencies = [
        "sites.Site",
        "intecomm_screening.patientlog",
    ]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Patient Followup Call"
        verbose_name_plural = "Patient Followup Calls"
        ordering = ["report_datetime"]



# Generated by Django 4.1.7 on 2023-05-10 00:43

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_prn",
            "0005_rename_historicaloffschedulefollowup_historicaloffscheduleinte_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicaloffschedulecomm",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule community-based integrated care",
                "verbose_name_plural": "historical Off-schedule community-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaloffscheduleinte",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule facility-based integrated care",
                "verbose_name_plural": "historical Off-schedule facility-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalonschedulecomm",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical On-schedule community-based integrated care",
                "verbose_name_plural": "historical On-schedule community-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalonscheduleinte",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical On-schedule facility-based integrated care",
                "verbose_name_plural": "historical On-schedule facility-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulecomm",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule community-based integrated care",
                "verbose_name_plural": "Off-schedule community-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="offscheduleinte",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule facility-based integrated care",
                "verbose_name_plural": "Off-schedule facility-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="onschedulecomm",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "On-schedule community-based integrated care",
                "verbose_name_plural": "On-schedule community-based integrated care",
            },
        ),
        migrations.AlterModelOptions(
            name="onscheduleinte",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "On-schedule facility-based integrated care",
                "verbose_name_plural": "On-schedule facility-based integrated care",
            },
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..forms import LocationUpdateForm
from ..models import LocationUpdate
from .modeladmin_mixins import CrfModelAdmin


@admin.register(LocationUpdate, site=intecomm_subject_admin)
class LocationUpdateAdmin(CrfModelAdmin):
    form = LocationUpdateForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Update", {"fields": ("location", "location_other", "comments", "next_location")}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "location": admin.VERTICAL,
        "next_location": admin.VERTICAL,
    }



from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import QuerySet
from edc_appointment.admin import AppointmentAdmin as BaseAdmin
from edc_appointment.admin_site import edc_appointment_admin
from edc_appointment.models import Appointment, AppointmentType
from edc_appointment.utils import get_allow_skipped_appt_using
from edc_constants.constants import HOSPITAL, NOT_APPLICABLE
from intecomm_rando.constants import FACILITY_ARM
from intecomm_rando.utils import get_assignment_for_subject

from ..forms import AppointmentForm

edc_appointment_admin.unregister(Appointment)


@admin.register(Appointment, site=edc_appointment_admin)
class AppointmentAdmin(BaseAdmin):
    form = AppointmentForm

    def allow_skipped_appointments(self, request) -> bool:
        """Return True only if the subject is in the facility arm."""
        allow = False
        if get_allow_skipped_appt_using():
            try:
                get_assignment_for_subject(
                    request.GET.get("subject_identifier")
                ) == FACILITY_ARM
            except ObjectDoesNotExist:
                pass
            else:
                allow = True
        return allow

    def get_appt_type_queryset(self, request) -> QuerySet:
        if not self.allow_skipped_appointments(request):
            return AppointmentType.objects.exclude(
                name__in=[HOSPITAL, NOT_APPLICABLE]
            ).order_by("display_index")
        return AppointmentType.objects.exclude(name__in=[HOSPITAL]).order_by("display_index")



# Generated by Django 4.2.3 on 2023-08-05 04:18

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0089_alter_healtheconomicshouseholdhead_hoh_education_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_age",
            field=models.IntegerField(
                help_text="In years. Note: if you are the household head, indicate your age as of today",
                validators=[
                    django.core.validators.MinValueValidator(18),
                    django.core.validators.MaxValueValidator(110),
                ],
                verbose_name="How old is the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_education",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.education",
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.employmenttype",
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.ethnicities",
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                help_text="Note: if you are the household head, indicate your gender",
                max_length=15,
                verbose_name="Is the household head female or male?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_insurance",
            field=models.ManyToManyField(
                help_text="Not applicable if patient is head of household",
                related_name="+",
                to="edc_he.insurancetypes",
                verbose_name="Household head’s health insurance and ‘club’ status ",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.religions",
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_age",
            field=models.IntegerField(
                help_text="In years. Note: if you are the household head, indicate your age as of today",
                validators=[
                    django.core.validators.MinValueValidator(18),
                    django.core.validators.MaxValueValidator(110),
                ],
                verbose_name="How old is the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_education",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.education",
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.employmenttype",
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.ethnicities",
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                help_text="Note: if you are the household head, indicate your gender",
                max_length=15,
                verbose_name="Is the household head female or male?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.religions",
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2023-01-23 15:48

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0006_alter_healtheconomics_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="alcohol_consumption",
            field=models.CharField(
                choices=[
                    ("occasionally", "Occasionally"),
                    ("1_2_per_week", "1-2 times a week"),
                    ("3_4_per_week", "3-4 times a week"),
                    ("daily", "Daily"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If yes, how often do you drink alcohol?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="alcohol_consumption",
            field=models.CharField(
                choices=[
                    ("occasionally", "Occasionally"),
                    ("1_2_per_week", "1-2 times a week"),
                    ("3_4_per_week", "3-4 times a week"),
                    ("daily", "Daily"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If yes, how often do you drink alcohol?",
            ),
        ),
    ]



from __future__ import annotations

from django.contrib import admin
from edc_action_item import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin
from edc_transfer.modeladmin_mixins import SubjectTransferModelAdminMixin

from ..admin_site import intecomm_prn_admin
from ..forms import SubjectTransferForm
from ..models import SubjectTransfer


@admin.register(SubjectTransfer, site=intecomm_prn_admin)
class SubjectTransferAdmin(
    SiteModelAdminMixin,
    SubjectTransferModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = SubjectTransferForm

    # TODO: remove with Django > 4.2.5
    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        self.list_filter = list_filter
        return list_filter



# Generated by Django 4.2.4 on 2023-08-28 17:10

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0013_subjectconsentug"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectconsentug",
            options={
                "verbose_name": "Subject Consent (Uganda)",
                "verbose_name_plural": "Subject Consents (Uganda)",
            },
        ),
    ]



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from edc_registration.models import RegisteredSubject

from intecomm_screening.models import SubjectScreening, SubjectScreeningUg

from .subject_consent import SubjectConsent
from .subject_consent_ug import SubjectConsentUg


class SubjectConsentDeleteError(Exception):
    pass


def update_subjectconsent(instance, subject_screening_model_cls=None):
    subject_screening = subject_screening_model_cls.objects.get(
        screening_identifier=instance.screening_identifier
    )
    subject_screening.subject_identifier = instance.subject_identifier
    subject_screening.consented = True
    subject_screening.save_base(update_fields=["subject_identifier", "consented"])

    subject_screening.patient_log.subject_identifier = instance.subject_identifier
    subject_screening.patient_log.consent_datetime = instance.consent_datetime
    subject_screening.patient_log.save_base(
        update_fields=["subject_identifier", "consent_datetime"]
    )
    # TODO: is this the right place to do this?
    rs_obj = RegisteredSubject.objects.get(subject_identifier=instance.subject_identifier)
    try:
        rs_obj.full_name = instance.full_name
    except AttributeError:
        pass
    try:
        rs_obj.familiar_name = instance.familiar_name
    except AttributeError:
        pass
    rs_obj.save(update_fields=["full_name", "familiar_name"])


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="update_subjectconsent_on_post_save",
)
def update_subjectconsent_on_post_save(sender, instance, raw, created, **kwargs):
    if not raw:
        if created:
            update_subjectconsent(instance, subject_screening_model_cls=SubjectScreening)


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsentUg,
    dispatch_uid="update_subjectconsentug_on_post_save",
)
def update_subjectconsentug_on_post_save(sender, instance, raw, created, **kwargs):
    if not raw:
        if created:
            update_subjectconsent(instance, subject_screening_model_cls=SubjectScreeningUg)


@receiver(
    pre_delete,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subjectconsent_on_pre_delete",
)
def subjectconsent_on_pre_delete(sender, instance, using, **kwargs):
    raise SubjectConsentDeleteError(
        f"Subject consent may not be deleted. Got {instance.subject_identifier}."
    )


@receiver(
    pre_delete,
    weak=False,
    sender=SubjectConsentUg,
    dispatch_uid="subjectconsentug_on_pre_delete",
)
def subjectconsentug_on_pre_delete(sender, instance, using, **kwargs):
    raise SubjectConsentDeleteError(
        f"Subject consent may not be deleted. Got {instance.subject_identifier}."
    )



from __future__ import annotations

import re
import urllib.parse
from typing import TYPE_CHECKING, Type

from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from edc_consent.utils import get_remove_patient_names_from_countries
from edc_constants.choices import GENDER
from edc_constants.constants import UUID_PATTERN
from edc_sites.admin import SiteModelAdminMixin

from intecomm_sites.sites import all_sites

from ...models import PatientGroup
from ..actions import render_pdf_action
from ..list_filters import (
    AttendDateListFilter,
    ConsentedListFilter,
    DxListFilter,
    InPatientGroup,
    LastApptListFilter,
    NextApptListFilter,
    ScreenedListFilter,
    StableListFilter,
)
from ..modeladmin_mixins import ChangeListTopBarModelAdminMixin
from ..patient_call_inlines import AddPatientCallInline, ViewPatientCallInline
from .change_list_template_context import ChangeListTemplateContext

if TYPE_CHECKING:
    from intecomm_consent.models import SubjectConsent, SubjectConsentUg

    from ...models import SubjectScreening, SubjectScreeningUg


class PatientLogModelAdminMixin(
    SiteModelAdminMixin,
    ChangeListTopBarModelAdminMixin,
):
    all_sites = all_sites

    autocomplete_fields = ["site"]
    inlines = [AddPatientCallInline, ViewPatientCallInline]
    actions = [render_pdf_action]
    list_per_page = 5
    show_object_tools = True
    show_cancel = True

    extra_pii_attrs = ["first_column"]

    change_list_note = format_html(
        "In addition to other values, you may search for patients on the last 4-digits of "
        "either their mobile number or hospital identifier."
    )
    change_list_help = (
        "Searches on encrypted data work on exact uppercase matches only. When "
        'searching on a full name, put the full name in quotations, for example, "JOHN SMITH".'
    )

    additional_instructions = format_html(
        "Only include patients that are known to have a qualifying "
        "condition and are stable in-care.<BR>"
        '<h3 style="color:orange;">Note:</h3> Log calls and call attempts at the bottom '
        "of this form."
    )

    list_display = (
        "first_column",
        "dx",
        "group_name",
        "screened",
        "appts",
        "contacts",
        "talks",
        "site_id",
        "user_created",
        "created",
        "modified",
        "screening_id",
        "subject_id",
        "site",
    )

    list_filter = (
        "report_datetime",
        "willing_to_screen",
        "call_attempts",
        InPatientGroup,
        DxListFilter,
        StableListFilter,
        ScreenedListFilter,
        ConsentedListFilter,
        AttendDateListFilter,
        NextApptListFilter,
        LastApptListFilter,
        "first_health_talk",
        "second_health_talk",
        "gender",
        "printed",
    )

    filter_horizontal = ("conditions",)

    radio_fields = {
        "first_health_talk": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "may_contact": admin.VERTICAL,
        "second_health_talk": admin.VERTICAL,
        "site": admin.VERTICAL,
        "stable": admin.VERTICAL,
        "screening_refusal_reason": admin.VERTICAL,
        "willing_to_screen": admin.VERTICAL,
    }

    readonly_fields = (
        "screening_identifier",
        "screening_datetime",
        "subject_identifier",
        "consent_datetime",
        "group_identifier",
        "filing_identifier",
        "patient_log_identifier",
        "comment",
    )

    @property
    def subject_screening_model_cls(self) -> Type[SubjectScreening, SubjectScreeningUg]:
        pass

    @property
    def subject_consent_model_cls(self) -> Type[SubjectConsent, SubjectConsentUg]:
        pass

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    @admin.display(description="Patient log", ordering="filing_identifier")
    def first_column(self, obj=None):
        legal_name = (
            "<unknown>" if re.match(UUID_PATTERN, str(obj.legal_name)) else obj.legal_name
        )
        context = dict(
            legal_name=legal_name,
            filing_identifier=obj.filing_identifier,
            last_4_hospital_identifier=obj.last_4_hospital_identifier,
            last_4_contact_number=obj.last_4_contact_number,
            initials=obj.initials,
            gender=obj.gender.upper(),
            age_in_years=obj.age_in_years,
        )
        for country in get_remove_patient_names_from_countries():
            if obj and obj.site.id in [s.site_id for s in self.all_sites.get(country)]:
                context.pop("legal_name")
                break
        return format_html(
            render_to_string(
                "intecomm_screening/change_list_patient_log_first_column.html",
                context=context,
            )
        )

    @admin.display(description="Date logged", ordering="report_datetime")
    def date_logged(self, obj=None):
        return obj.report_datetime.date()

    @admin.display(description="next_appt", ordering="next_appt_date")
    def next_appt(self, obj=None):
        return obj.next_appt_date

    @admin.display(description="last_appt", ordering="last_appt_date")
    def last_appt(self, obj=None):
        return obj.last_appt_date

    @admin.display(description="FILE", ordering="filing_identifier")
    def filing_id(self, obj=None):
        context = dict(filing_identifier=obj.filing_identifier)
        return format_html(
            render_to_string(
                "intecomm_screening/change_list_filing_identifier.html", context=context
            )
        )

    @admin.display(description="Screening ID", ordering="screening_identifier")
    def screening_id(self, obj=None):
        return (
            None
            if re.match(UUID_PATTERN, obj.screening_identifier)
            else obj.screening_identifier
        )

    @admin.display(description="Subject ID", ordering="subject_identifier")
    def subject_id(self, obj=None):
        return (
            None if re.match(UUID_PATTERN, obj.subject_identifier) else obj.subject_identifier
        )

    @admin.display(description="DX")
    def dx(self, obj=None):
        context = dict(
            diagnoses=[c.name.upper() for c in obj.conditions.all().order_by("name")]
        )
        return format_html(
            render_to_string("intecomm_screening/change_list_dx.html", context=context)
        )

    @admin.display(description="Appts", ordering="next_appt_date")
    def appts(self, obj=None):
        context = dict(
            last_appt=obj.last_appt_date or "-",
            next_appt=obj.next_appt_date or "-",
        )
        return format_html(
            render_to_string("intecomm_screening/change_list_appts.html", context=context)
        )

    @admin.display(description="Contacts", ordering="contact_number")
    def contacts(self, obj=None):
        add_patient_call_url = reverse(
            "intecomm_screening_admin:intecomm_screening_patientcall_add"
        )
        patient_call_url = reverse(
            "intecomm_screening_admin:intecomm_screening_patientcall_changelist"
        )
        patient_call_url = f"{patient_call_url}?q={obj.id}"
        context = dict(
            patient_log_id=obj.id,
            add_patient_call_url=add_patient_call_url,
            patient_call_url=patient_call_url,
            contact_number=obj.contact_number,
            alt_contact_number=obj.alt_contact_number,
            call_attempts=obj.call_attempts,
        )
        return format_html(
            render_to_string("intecomm_screening/change_list_contacts.html", context=context)
        )

    @admin.display(description="Talks")
    def talks(self, obj=None):
        context = dict(
            ht1=obj.first_health_talk,
            ht2=obj.second_health_talk,
        )
        return format_html(
            render_to_string("intecomm_screening/change_list_talks.html", context=context)
        )

    @admin.display(description="Site", ordering="site")
    def site_id(self, obj=None):
        return obj.site.id

    @admin.display(description="Patient", ordering="familiar_name")
    def patient(self, obj=None):
        return f"{obj.familiar_name} ({obj.initials})"

    @admin.display(description="Log/Scr/Consent", ordering="screening_datetime")
    def screened(self, obj=None):
        return format_html(
            render_to_string(
                "intecomm_screening/change_list_screen_and_consent.html",
                context=self.get_screen_and_consent_template_context(obj),
            )
        )

    @admin.display(description="Log/Scr/Consent", ordering="screening_datetime")
    def screened_no_links(self, obj=None):
        return format_html(
            render_to_string(
                "intecomm_screening/change_list_screen_and_consent.html",
                context=self.get_screen_and_consent_template_context(obj),
            )
        )

    @admin.display(description="Consented", ordering="consent_datetime")
    def consented(self, obj=None):
        if obj.subject_identifier:
            return obj.consent_datetime.date()
        return None

    @admin.display(description="Group", ordering="patientgroup__name")
    def group_name(self, obj=None):
        context = dict()
        if obj.patientgroup_set.all().count() > 0:
            patient_group = obj.patientgroup_set.all().first()
            patient_group_url = reverse(
                "intecomm_screening_admin:intecomm_screening_patientgroup_changelist"
            )
            patient_group_url = (
                f"{patient_group_url}?q={urllib.parse.quote(patient_group.name)}"
            )
            context.update(
                patient_group_url=patient_group_url,
                patient_group_name=patient_group.name.replace(" ", "\n"),
                subject_identifier=obj.subject_identifier,
                subject_dashboard_url=self.get_subject_dashboard_url(obj),
            )
        return format_html(
            render_to_string("intecomm_screening/change_list_group.html", context=context)
        )

    @admin.display(description="Calls", ordering="contact_attempts")
    def calls(self, obj):
        url = reverse("intecomm_screening_admin:intecomm_screening_patientcall_changelist")
        url = f"{url}?q={obj.patientcall.id}"
        return format_html(f'<A href="{url}">{obj.contact_attempts}</a>')

    def get_search_results(self, request, queryset, search_term):
        """Union initial search queryset (qs1) with
        patients in a group whose name matches the search term (qs2).

        Note: queryset is a queryset already passed through the
        changelist's filters.
        """
        qs1, may_have_duplicates = super().get_search_results(
            request,
            queryset,
            search_term,
        )
        try:
            patient_group = PatientGroup.on_site.prefetch_related("patients").get(
                name__iexact=search_term
            )
        except ObjectDoesNotExist:
            qs = qs1
        else:
            pks = [v[0] for v in queryset.values_list("id")]
            pks = [v[0] for v in patient_group.patients.filter(pk__in=pks).values_list("id")]
            qs = qs1 | self.model.on_site.filter(id__in=pks)
        return qs, True

    def get_screen_and_consent_template_context(self, obj) -> dict:
        """Context for change_list_screen_and_consent.html"""
        return ChangeListTemplateContext(
            obj,
            subject_screening_model_cls=self.subject_screening_model_cls,
            subject_consent_model_cls=self.subject_consent_model_cls,
        ).context

    @staticmethod
    def get_subject_dashboard_url(obj):
        subject_dashboard_url = None
        if obj.subject_identifier and obj.group_identifier:
            subject_dashboard_url = reverse(
                "intecomm_dashboard:subject_dashboard_url",
                args=(obj.subject_identifier,),
            )
        return subject_dashboard_url

    def report(self, obj=None):
        return format_html(
            render_to_string(
                "intecomm_screening/change_list_screen_and_consent.html",
                context=self.get_screen_and_consent_template_context(obj),
            )
        )

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "gender":
            kwargs["choices"] = GENDER
        return super().formfield_for_choice_field(db_field, request, **kwargs)



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin.history import SimpleHistoryAdmin
from edc_prn.modeladmin_mixins import PrnModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_group_admin
from ..forms import CommunityCareLocationForm
from ..models import CommunityCareLocation


@admin.register(CommunityCareLocation, site=intecomm_group_admin)
class CommunityCareLocationAdmin(
    SiteModelAdminMixin,
    PrnModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = CommunityCareLocationForm

    date_hierarchy = "report_datetime"

    show_object_tools = True
    change_list_template: str = "intecomm_group/admin/communitycarelocation_change_list.html"

    fieldsets = (
        [
            None,
            {"fields": ("report_datetime",)},
        ],
        [
            "Description",
            {
                "fields": (
                    "name",
                    "location_type",
                    "location_type_other",
                    "opening_date",
                    "closing_date",
                    "description",
                ),
            },
        ],
        [
            "GPS (if available)",
            {
                "description": (
                    "Using google maps, right mouse click on the map point to copy to "
                    "clipboard, Paste here as is"
                ),
                "fields": (
                    "gps",
                    "latitude",
                    "longitude",
                ),
            },
        ],
        [
            "Notes",
            {"fields": ("notes",)},
        ],
        audit_fieldset_tuple,
    )

    list_display = ("__str__", "status", "opening_date", "closing_date", "meetings", "map")

    list_filter = ("opening_date", "closing_date")

    radio_fields = {
        "location_type": admin.VERTICAL,
    }

    readonly_fields = ("latitude", "longitude")

    search_fields = ("name",)

    @admin.display(description="Map")
    def map(self, obj=None):
        if obj.latitude and obj.longitude:
            return format_html(
                f'<A href="https://www.google.com/maps/@{obj.latitude},{obj.longitude},15z">'
                '<i class="fas fa-location-dot"></i>'
            )
        return None

    @admin.display(description="Meetings")
    def meetings(self, obj=None):
        url = reverse("intecomm_group_admin:intecomm_group_patientgroupappointment_changelist")
        search_term = "+".join(obj.name.split(" "))
        name = "&nbsp;".join(obj.name.split(" "))
        return format_html(f'<a href="{url}?q={search_term}">@{name}</a>')



# Generated by Django 4.1.7 on 2023-05-11 23:08

from django.db import migrations
import edc_sites.model_mixins
import intecomm_group.models.patient_group


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0012_alter_historicalpatientgroup_status_and_more"),
        ("intecomm_screening", "0031_alter_historicalpatientlog_next_appt_date_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="PatientGroupRando",
            fields=[],
            options={
                "verbose_name": "Patient group randomization",
                "verbose_name_plural": "Patient group randomization",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("intecomm_group.patientgroup",),
            managers=[
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
                ("objects", intecomm_group.models.patient_group.PatientGroupManager()),
            ],
        ),
    ]



from edc_appointment.constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    NEW_APPT,
)
from edc_constants.constants import CLOSED, COMPLETE, NEW, OPEN
from edc_visit_tracking.constants import SCHEDULED
from intecomm_form_validators import DISSOLVED, IN_FOLLOWUP, RECRUITING

from .constants import ATTENDED

GROUP_STATUS_CHOICES = (
    (NEW, "New"),
    (RECRUITING, "Recruiting"),
    (COMPLETE, "Complete/Ready"),
    (IN_FOLLOWUP, "In followup"),
    (DISSOLVED, "Dissolved"),
)

LOCATION_STATUS = ((OPEN, "Open"), (CLOSED, "Closed"))

MEETING_STATUS = (
    (SCHEDULED, "Scheduled"),
    (ATTENDED, "Attended"),
)

APPT_STATUS = (
    (NEW_APPT, "Scheduled"),
    (IN_PROGRESS_APPT, "In Progress"),
    (COMPLETE_APPT, "Done"),
    (CANCELLED_APPT, "Cancelled"),
)



from django.db import models
from django_crypto_fields.fields import EncryptedTextField
from edc_constants.choices import YES_NO
from edc_constants.constants import CLINIC, COMMUNITY, OTHER
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from intecomm_subject.model_mixins import CrfModelMixin


class LocationUpdate(CrfModelMixin, BaseUuidModel):
    location = models.CharField(
        verbose_name="Where is this visit taking place",
        choices=(
            (COMMUNITY, "In the community"),
            (CLINIC, "At the facility"),
            (OTHER, "Other, specify below ..."),
        ),
        max_length=100,
    )

    location_other = OtherCharField()

    comments = EncryptedTextField(
        verbose_name=(
            "Briefly explain why the participant is not at the location expected "
            "based on their randomization"
        ),
    )

    next_location = models.CharField(
        verbose_name=(
            "Is the participant expected to attend their next visit at the "
            "location they were randomized to"
        ),
        choices=YES_NO,
        max_length=15,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Location update"
        verbose_name_plural = "Location updates"



# Generated by Django 4.2.1 on 2023-05-24 02:04

from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0046_alter_healtheconomicsassets_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="cooking_fuel_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER cooking fuel, specify",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="floor_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER floor material, specify",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="lighting_source_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER lighting source, specify",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="roof_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER roof material, specify",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="toilet_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER type of toilet, specify",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsassets",
            name="water_source_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER water source, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="cooking_fuel_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER cooking fuel, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="floor_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER floor material, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="lighting_source_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER lighting source, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="roof_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER roof material, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="toilet_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER type of toilet, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsassets",
            name="water_source_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER water source, specify",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="external_wall_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER external wall material, specify",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="external_window_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER external window material, specify",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="water_source",
            field=models.CharField(
                choices=[
                    ("piped_into_plot", "Piped into dwelling/yard plot"),
                    ("piped_to_neighbour", "Piped to neighbour"),
                    ("standpipe", "Public tap/standpipe"),
                    ("borehole", "Tube well or borehole"),
                    ("protected_well", "Protected dug well"),
                    ("protected_spring", "Protected spring"),
                    ("rainwater", "Rainwater"),
                    (
                        "bottled_water_improved",
                        "Bottled water, improved source for cooking/hand washing (1-7)",
                    ),
                    ("unprotected_well", "Unprotected dug well"),
                    ("unprotected_spring", "Unprotected spring"),
                    ("tanker", "Tanker truck/cart with small tank"),
                    ("surface_water", "Surface water (river etc.)"),
                    (
                        "bottled_water_unimproved",
                        "Bottle water, unimproved source for cooking/hand washing (9-12)",
                    ),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the main source of drinking water for the household?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="external_wall_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER external wall material, specify",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="external_window_material_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER external window material, specify",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="water_source",
            field=models.CharField(
                choices=[
                    ("piped_into_plot", "Piped into dwelling/yard plot"),
                    ("piped_to_neighbour", "Piped to neighbour"),
                    ("standpipe", "Public tap/standpipe"),
                    ("borehole", "Tube well or borehole"),
                    ("protected_well", "Protected dug well"),
                    ("protected_spring", "Protected spring"),
                    ("rainwater", "Rainwater"),
                    (
                        "bottled_water_improved",
                        "Bottled water, improved source for cooking/hand washing (1-7)",
                    ),
                    ("unprotected_well", "Unprotected dug well"),
                    ("unprotected_spring", "Unprotected spring"),
                    ("tanker", "Tanker truck/cart with small tank"),
                    ("surface_water", "Surface water (river etc.)"),
                    (
                        "bottled_water_unimproved",
                        "Bottle water, unimproved source for cooking/hand washing (9-12)",
                    ),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the main source of drinking water for the household?",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-04-06 19:10

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0024_historicalhtninitialreview_med_start_date_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="dminitialreview",
            old_name="med_start_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="dminitialreview",
            old_name="dx_estimated_date",
            new_name="dx_calculated_date",
        ),
        migrations.RenameField(
            model_name="historicaldminitialreview",
            old_name="dx_estimated_date",
            new_name="dx_calculated_date",
        ),
        migrations.RenameField(
            model_name="historicaldminitialreview",
            old_name="med_start_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="hivinitialreview",
            old_name="arv_initiated",
            new_name="rx_init",
        ),
        migrations.RenameField(
            model_name="hivinitialreview",
            old_name="arv_initiation_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="hivinitialreview",
            old_name="arv_initiation_actual_date",
            new_name="rx_init_date",
        ),
        migrations.RenameField(
            model_name="historicalhivinitialreview",
            old_name="dx_estimated_date",
            new_name="dx_calculated_date",
        ),
        migrations.RenameField(
            model_name="historicalhivinitialreview",
            old_name="arv_initiated",
            new_name="rx_init",
        ),
        migrations.RenameField(
            model_name="historicalhivinitialreview",
            old_name="arv_initiation_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="historicalhivinitialreview",
            old_name="arv_initiation_actual_date",
            new_name="rx_init_date",
        ),
        migrations.RenameField(
            model_name="htninitialreview",
            old_name="med_start_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="historicalhtninitialreview",
            old_name="med_start_ago",
            new_name="rx_init_ago",
        ),
        migrations.RenameField(
            model_name="historicalhtninitialreview",
            old_name="dx_estimated_date",
            new_name="dx_calculated_date",
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-05 04:13

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0039_icecapa_historicalicecapa_historicaleq5d3l_eq5d3l_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalhivmedicationadherence",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Medication Adherence (HIV)",
                "verbose_name_plural": "historical Medication Adherence (HIV)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhtnmedicationadherence",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Medication Adherence (Hypertension)",
                "verbose_name_plural": "historical Medication Adherence (Hypertension)",
            },
        ),
        migrations.AlterModelOptions(
            name="hivmedicationadherence",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Medication Adherence (HIV)",
                "verbose_name_plural": "Medication Adherence (HIV)",
            },
        ),
        migrations.AlterModelOptions(
            name="htnmedicationadherence",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Medication Adherence (Hypertension)",
                "verbose_name_plural": "Medication Adherence (Hypertension)",
            },
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-26 18:14

import edc_facility.model_mixins
import edc_sites.model_mixins
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        # ("intecomm_facility", "0002_alter_healthfacility_device_created_and_more"),
        ("intecomm_screening", "0052_auto_20230726_1634"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="IdenfifierFormat",
            new_name="IdentifierFormat",
        ),
        migrations.CreateModel(
            name="HealthFacility",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("intecomm_facility.healthfacility",),
            managers=[
                ("objects", edc_facility.model_mixins.Manager()),
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
            ],
        ),
    ]



from edc_model.models import BaseUuidModel
from edc_qol.model_mixins import Eq5d3lModelMixin

from ..model_mixins import CrfModelMixin


class Eq5d3l(
    Eq5d3lModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    class Meta(Eq5d3lModelMixin.Meta, CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "EuroQol EQ-5D-3L Instrument"
        verbose_name_plural = "EuroQol EQ-5D-3L Instrument"



from copy import deepcopy

from edc_sites import get_current_country

from intecomm_screening.constants import UGANDA


class IntecommAdminSiteMixin:
    @staticmethod
    def get_hidden_models(request) -> dict:
        return {}

    def get_app_list(self, request, app_label=None):
        app_list = super().get_app_list(request, app_label=app_label)
        if get_current_country(request=request) == UGANDA:
            app_list_copy = deepcopy(app_list)
            for index, item in enumerate(app_list_copy):
                for app_label, object_names in self.get_hidden_models(request).items():
                    if item["app_label"] == app_label:
                        for i, model in enumerate(item["models"]):
                            for object_name in object_names:
                                if model["object_name"] == object_name:
                                    app_list[index]["models"].remove(model)
        return app_list



# Generated by Django 4.2.4 on 2023-08-26 22:49

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0113_alter_htnreview_device_created_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dmmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
        migrations.AlterField(
            model_name="familyhistory",
            name="dm_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="familyhistory",
            name="hiv_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has HIV?",
            ),
        ),
        migrations.AlterField(
            model_name="familyhistory",
            name="htn_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has high blood pressure?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfamilyhistory",
            name="dm_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfamilyhistory",
            name="hiv_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has HIV?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfamilyhistory",
            name="htn_in_household",
            field=models.CharField(
                choices=[
                    ("No", "No"),
                    ("yes_spouse", "Yes, my spouse"),
                    ("yes_parents", "Yes, one of my parents living with me"),
                    ("yes_relative", "Yes, another relative living with me"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Do you know if anyone else in your household has high blood pressure?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtnmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
        migrations.AlterField(
            model_name="hivmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
        migrations.AlterField(
            model_name="htnmedicationadherence",
            name="visual_score_confirmed",
            field=models.IntegerField(
                help_text="%",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.constants import CLOSED, OPEN
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField
from edc_sites.models import SiteModelMixin

from intecomm_lists.models import LocationTypes

from ..choices import LOCATION_STATUS


class CommunityCareLocation(SiteModelMixin, BaseUuidModel):
    report_datetime = models.DateTimeField()

    name = models.CharField(max_length=25, unique=True, null=True, blank=False)

    opening_date = models.DateField()

    closing_date = models.DateField(
        null=True,
        blank=True,
        help_text=(
            "Complete the 'closing' date if the location is no longer being used by any group"
        ),
    )

    status = models.CharField(max_length=25, choices=LOCATION_STATUS, default=OPEN)

    location_type = models.ForeignKey(LocationTypes, on_delete=models.PROTECT)

    location_type_other = OtherCharField()

    gps = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        help_text="copy and paste directly from google maps",
    )

    latitude = models.FloatField(
        validators=[MinValueValidator(-90), MaxValueValidator(90)],
        null=True,
        blank=True,
        help_text="in degrees. copy and paste directly from google maps",
    )

    longitude = models.FloatField(
        validators=[MinValueValidator(-180), MaxValueValidator(180)],
        null=True,
        blank=True,
        help_text="in degrees. copy and paste directly from google maps",
    )

    description = models.TextField(null=True, blank=True)

    notes = models.TextField(null=True, blank=True)

    def __str__(self):
        return self.name.title()

    def save(self, *args, **kwargs):
        self.status = CLOSED if self.opening_date and self.closing_date else OPEN
        if self.gps:
            lat, lon = self.gps.split(",")
            self.latitude = float(lat.strip())
            self.longitude = float(lon.strip())
        super().save(*args, **kwargs)

    class Meta(SiteModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Community Care Location"
        verbose_name_plural = "Community Care Locations"



from django.urls.conf import path, re_path
from django.views.generic import RedirectView

from .views import GroupManagementView

app_name = "intecomm_screening"
urlpatterns = [
    re_path(
        r"^group_managment/(?P<ct>\d+)/"
        r"(?P<ids>([a-f0-9]{8}-?[a-f0-9]{4}-?4[a-f0-9]{3}-?[89ab][a-f0-9]{3}-?[a-f0-9]{12})(,\s*[a-f0-9]{8}-?[a-f0-9]{4}-?4[a-f0-9]{3}-?[89ab][a-f0-9]{3}-?[a-f0-9]{12})*)/$",  # noqa
        GroupManagementView.as_view(),
        name="group_managment_url",
    ),
    path("", RedirectView.as_view(url=f"/{app_name}/admin/"), name="home_url"),
]



# Generated by Django 4.2.1 on 2023-05-23 22:28

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0041_health_economics_assets_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsins",
            name="fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Are you the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_age",
            field=models.IntegerField(
                help_text="In years",
                validators=[
                    django.core.validators.MinValueValidator(18),
                    django.core.validators.MaxValueValidator(110),
                ],
                verbose_name="How old is the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                max_length=15,
                verbose_name="Is the household head female or male?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="relationship_to_hoh",
            field=models.CharField(
                choices=[
                    ("WIFE_HUSBAND", "Wife/Husband"),
                    ("SON_DAUGHTER", "Son/Daughter"),
                    ("SON_DAUGHTERINLAW", "Son/Daughter-in-law"),
                    ("GRANDCHILD", "Grandchild"),
                    ("PARENT", "Parent"),
                    ("PARENTINLAW", "Parent-in-law"),
                    ("BROTHER_SISTER", "Brother/Sister"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your relationship to the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Are you the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_age",
            field=models.IntegerField(
                help_text="In years",
                validators=[
                    django.core.validators.MinValueValidator(18),
                    django.core.validators.MaxValueValidator(110),
                ],
                verbose_name="How old is the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")],
                max_length=15,
                verbose_name="Is the household head female or male?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="relationship_to_hoh",
            field=models.CharField(
                choices=[
                    ("WIFE_HUSBAND", "Wife/Husband"),
                    ("SON_DAUGHTER", "Son/Daughter"),
                    ("SON_DAUGHTERINLAW", "Son/Daughter-in-law"),
                    ("GRANDCHILD", "Grandchild"),
                    ("PARENT", "Parent"),
                    ("PARENTINLAW", "Parent-in-law"),
                    ("BROTHER_SISTER", "Brother/Sister"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your relationship to the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalicecapa",
            name="enjoyment",
            field=models.CharField(
                choices=[
                    ("4", "I can have <B>a lot</B> of enjoyment and pleasure"),
                    ("3", "I can have <B>quite a lot</B> of enjoyment and pleasure"),
                    ("2", "I can have <B>a little</B> enjoyment and pleasure"),
                    ("1", "I <B>cannot</B> have <B>any</B> enjoyment and pleasure"),
                ],
                max_length=5,
                verbose_name="Enjoyment and pleasure",
            ),
        ),
        migrations.AlterField(
            model_name="icecapa",
            name="enjoyment",
            field=models.CharField(
                choices=[
                    ("4", "I can have <B>a lot</B> of enjoyment and pleasure"),
                    ("3", "I can have <B>quite a lot</B> of enjoyment and pleasure"),
                    ("2", "I can have <B>a little</B> enjoyment and pleasure"),
                    ("1", "I <B>cannot</B> have <B>any</B> enjoyment and pleasure"),
                ],
                max_length=5,
                verbose_name="Enjoyment and pleasure",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2023-02-13 15:12

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0013_alter_bloodresultsfbc_results_abnormal_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="activity_combined_mn_avg_day",
            field=models.IntegerField(
                blank=True,
                help_text="Number of minutes in a typical day",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(1080),
                ],
                verbose_name="On an average day, how many MINUTES do you spend on all physical activities combined? (moderate or vigorous-intensity work activities, exercise, sports and fitness)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="activity_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="See interviewer notes above.",
                max_length=15,
                null=True,
                verbose_name="Does your work involve moderate or vigorous-intensity physical activity?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="smoker_current_duration_estimated_date",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="Estimated date smoker has been smoking since?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="smoking_status",
            field=models.CharField(
                choices=[
                    ("smoker", "Currently smoke"),
                    ("former_smoker", "Used to smoke but stopped"),
                    ("nonsmoker", "Never smoked"),
                ],
                max_length=15,
                verbose_name="Which of these options describes you?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="activity_combined_mn_avg_day",
            field=models.IntegerField(
                blank=True,
                help_text="Number of minutes in a typical day",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(1080),
                ],
                verbose_name="On an average day, how many MINUTES do you spend on all physical activities combined? (moderate or vigorous-intensity work activities, exercise, sports and fitness)",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="activity_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="See interviewer notes above.",
                max_length=15,
                null=True,
                verbose_name="Does your work involve moderate or vigorous-intensity physical activity?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="smoker_current_duration_estimated_date",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="Estimated date smoker has been smoking since?",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="smoking_status",
            field=models.CharField(
                choices=[
                    ("smoker", "Currently smoke"),
                    ("former_smoker", "Used to smoke but stopped"),
                    ("nonsmoker", "Never smoked"),
                ],
                max_length=15,
                verbose_name="Which of these options describes you?",
            ),
        ),
    ]



from __future__ import annotations

from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms import OffScheduleCommForm
from ..models import OffScheduleComm
from .modeladmin_mixins import OffScheduleAdmin


@admin.register(OffScheduleComm, site=intecomm_prn_admin)
class OffScheduleCommAdmin(OffScheduleAdmin):
    form = OffScheduleCommForm

    # TODO: remove with Django > 4.2.5
    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        self.list_filter = list_filter
        return list_filter



import re
from tempfile import mkdtemp

from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from edc_auth.auth_objects import PII, PII_VIEW
from edc_consent.utils import get_remove_patient_names_from_countries
from edc_constants.constants import UUID_PATTERN
from edc_identifier.utils import convert_to_human_readable
from edc_pdf_reports import Report
from edc_sites import get_sites_by_country
from edc_utils import convert_php_dateformat
from reportlab.lib import colors
from reportlab.lib.enums import TA_CENTER, TA_LEFT, TA_RIGHT
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import (
    ParagraphStyle,
    StyleSheet1,
    _baseFontNameB,
    _baseFontNameI,
)
from reportlab.lib.units import cm
from reportlab.platypus import Paragraph, Spacer, Table, TableStyle

from intecomm_consent.models import SubjectConsent
from intecomm_screening.models import SubjectScreening


class PatientLogReportError(Exception):
    pass


class PatientLogReport(Report):
    default_page = dict(
        rightMargin=1 * cm,
        leftMargin=1 * cm,
        topMargin=1.5 * cm,
        bottomMargin=1.5 * cm,
        pagesize=A4,
    )

    def __init__(self, patient_log=None, user: User = None, **kwargs):
        super().__init__(**kwargs)
        self.object = patient_log
        if not patient_log.filing_identifier or re.match(
            UUID_PATTERN, patient_log.filing_identifier
        ):
            patient_log.save()
        if not patient_log.patient_log_identifier or re.match(
            UUID_PATTERN, patient_log.patient_log_identifier
        ):
            patient_log.save()
        self.user = user  # a User model instance
        if not user.groups.filter(name=PII):
            raise PatientLogReportError("User does not have permissions to access PII")
        self.image_folder = mkdtemp()
        self.report_filename = f"{self.object.filing_identifier}.pdf"

    def screening(self, attr):
        try:
            obj = SubjectScreening.objects.get(patient_log=self.object)
        except ObjectDoesNotExist:
            value = None
        else:
            value = getattr(obj, attr)
        return value

    def screening_date(self):
        value = self.screening("created")
        if value:
            value = value.strftime(convert_php_dateformat(settings.SHORT_DATETIME_FORMAT))
        return value

    def consent(self, attr):
        try:
            obj = SubjectConsent.objects.get(
                screening_identifier=self.screening("screening_identifier")
            )
        except ObjectDoesNotExist:
            value = None
        else:
            value = getattr(obj, attr)
        return value

    def consent_date(self):
        value = self.consent("created")
        if value:
            value = value.strftime(convert_php_dateformat(settings.SHORT_DATETIME_FORMAT))
        return value

    def add_to_styles(self, styles: StyleSheet1) -> StyleSheet1:
        styles.add(
            ParagraphStyle(name="confidential", fontSize=12, leading=11, alignment=TA_CENTER)
        )
        styles.add(
            ParagraphStyle(name="report_title", fontSize=16, leading=16, alignment=TA_RIGHT)
        )
        styles.add(
            ParagraphStyle(name="field_name_large", alignment=TA_LEFT, fontSize=11, leading=14)
        )
        styles.add(
            ParagraphStyle(
                name="section_heading",
                alignment=TA_LEFT,
                fontSize=11,
                leading=11,
                fontName=_baseFontNameB,
            )
        )
        styles.add(
            ParagraphStyle(
                name="row_instructions",
                fontSize=6,
                leading=7,
                alignment=TA_LEFT,
                fontName=_baseFontNameI,
            )
        )
        styles.add(
            ParagraphStyle(
                name="section_instructions",
                fontSize=10,
                # leading=7,
                alignment=TA_LEFT,
                fontName=_baseFontNameI,
            )
        )

        return styles

    def get_report_story(self, **kwargs):
        story = []
        data = [
            [Paragraph("CONFIDENTIAL", self.styles["confidential"])],
            [Paragraph(" ", self.styles["confidential"])],
            [Paragraph("INTECOMM Patient Reference", self.styles["report_title"])],
        ]
        t = Table(data, colWidths=(9 * cm))
        t.hAlign = "CENTER"
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            [
                Paragraph(
                    (
                        "KEEP THIS DOCUMENT IN A SECURE LOCATION. FILE SEQUENTIALLY "
                        "ACCORDING TO THE FILE NUMBER. DO NOT WRITE ANY PATIENT HEALTH "
                        "DATA ON THIS DOCUMENT."
                    ),
                    self.styles["line_data_mediumB"],
                ),
                Paragraph("", self.styles["line_label"]),
                [
                    Paragraph("EDC FILE NUMBER:", self.styles["field_name_large"]),
                    Paragraph(
                        self.object.filing_identifier or "", self.styles["field_name_large"]
                    ),
                ],
            ],
        ]
        t = Table(data, colWidths=(8 * cm, None, None), rowHeights=(1.5 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("INNERGRID", (1, 0), (-1, -1), 0.25, colors.black),
                    ("BOX", (2, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "TOP"),
                ]
            )
        )
        t.hAlign = "LEFT"
        story.append(t)

        data = [
            [
                Paragraph("", self.styles["line_data_mediumB"]),
                Paragraph("", self.styles["line_label"]),
                [
                    Paragraph("EDC SITE:", self.styles["field_name_large"]),
                    Paragraph(str(self.object.site_id) or "", self.styles["field_name_large"]),
                ],
            ],
        ]
        t = Table(data, colWidths=(8 * cm, None, None), rowHeights=(1.5 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("INNERGRID", (1, 0), (-1, -1), 0.25, colors.black),
                    ("BOX", (2, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "TOP"),
                ]
            )
        )
        t.hAlign = "LEFT"
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.25 * cm))

        data = [
            [Paragraph("SECTION ONE", self.styles["section_heading"])],
        ]
        t = Table(data, colWidths=(9 * cm))
        t.hAlign = "LEFT"
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            (
                [
                    Paragraph("FULL PATIENT NAME:", self.styles["line_data_large"]),
                    Paragraph(
                        "Not available in the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [Paragraph(self.legal_name or " ", self.styles["line_data_largest"])],
            ),
            (
                [
                    Paragraph("FAMILIAR NAME:", self.styles["line_data_large"]),
                    Paragraph(
                        "Not available in the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [Paragraph(self.familiar_name or " ", self.styles["line_data_largest"])],
            ),
            (
                [Paragraph("INITIALS (2-3 letters):", self.styles["line_data_large"])],
                [Paragraph(self.object.initials or " ", self.styles["line_data_largest"])],
            ),
            (
                [Paragraph("GENDER/AGE:", self.styles["line_data_large"])],
                [
                    Paragraph(
                        f"{self.object.gender or '?'}{self.object.age_in_years or '?'}",
                        self.styles["line_data_largest"],
                    )
                ],
            ),
            (
                [Paragraph("CONTACT:", self.styles["line_data_large"])],
                [
                    Paragraph(
                        (str(self.object.contact_number) or " ")
                        + (
                            f" / {str(self.object.alt_contact_number)}"
                            if str(self.object.alt_contact_number)
                            else ""
                        ),
                        self.styles["line_data_largest"],
                    )
                ],
            ),
            (
                [Paragraph("HOSPITAL IDENTIFIER:", self.styles["line_data_large"])],
                [
                    Paragraph(
                        str(self.object.hospital_identifier) or " ",
                        self.styles["line_data_largest"],
                    )
                ],
            ),
        ]
        t = Table(data, colWidths=(6 * cm, None), rowHeights=(1 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("INNERGRID", (0, 0), (1, 0), 0.25, colors.black),
                    ("INNERGRID", (0, 0), (1, 5), 0.25, colors.black),
                    ("BOX", (0, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
                ]
            )
        )
        story.append(t)
        data = [
            (
                [
                    Paragraph("Staff name:", self.styles["line_label"]),
                    Paragraph(self.object.user_created, self.styles["line_label"]),
                ],
                [Paragraph("Staff signature:", self.styles["line_label"])],
                [
                    Paragraph("Date completed:", self.styles["line_label"]),
                    Paragraph(
                        self.object.created.strftime(
                            convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                        ),
                        self.styles["line_label"],
                    ),
                ],
            ),
        ]
        t = Table(data, rowHeights=(1 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("BOX", (0, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
                ]
            )
        )
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            [Paragraph("SECTION TWO", self.styles["section_heading"])],
        ]
        t = Table(data, None)
        t.hAlign = "LEFT"
        story.append(t)

        data = [
            [
                Paragraph(
                    "Transcribe the identifiers below from the EDC as they become available.",
                    self.styles["section_instructions"],
                )
            ],
        ]
        t = Table(data)
        t.hAlign = "LEFT"
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            (
                [
                    Paragraph(
                        "PATIENT LOG IDENTIFIER:",
                        self.styles["line_data_large"],
                    ),
                    Paragraph(
                        "Complete once patient is added to the PATIENT LOG on the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [
                    Paragraph(
                        convert_to_human_readable(self.object.patient_log_identifier) or " ",
                        self.styles["line_data_largest"],
                    )
                ],
                [
                    Paragraph("Initials and date", self.styles["line_label"]),
                    Paragraph(
                        self.object.user_created,
                        self.styles["line_label"],
                    ),
                    Paragraph(
                        self.object.created.strftime(
                            convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                        ),
                        self.styles["line_label"],
                    ),
                ],
            ),
            (
                [
                    Paragraph("SCREENING IDENTIFIER:", self.styles["line_data_large"]),
                    Paragraph(
                        "Complete once subject's SCREENING FORM is submitted on the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [
                    Paragraph(
                        convert_to_human_readable(self.object.screening_identifier) or " ",
                        self.styles["line_data_largest"],
                    )
                ],
                [
                    Paragraph("Initials and date", self.styles["line_label"]),
                ],
            ),
            (
                [
                    Paragraph("SUBJECT IDENTIFIER:", self.styles["line_data_large"]),
                    Paragraph(
                        "Complete once subject's INFORMED CONSENT FORM is submitted "
                        "on the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [
                    Paragraph(
                        self.object.subject_identifier or " ", self.styles["line_data_largest"]
                    )
                ],
                [Paragraph("Initials and date", self.styles["line_label"])],
            ),
            (
                [
                    Paragraph("GROUP IDENTIFIER:", self.styles["line_data_large"]),
                    Paragraph(
                        "Complete once participant's GROUP has randomized on the EDC",
                        self.styles["row_instructions"],
                    ),
                ],
                [
                    Paragraph(
                        self.object.group_identifier or " ", self.styles["line_data_largest"]
                    )
                ],
                [Paragraph("Initials and date", self.styles["line_label"])],
            ),
        ]
        t = Table(data, colWidths=(6 * cm, None, 3 * cm), rowHeights=(1.5 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("INNERGRID", (0, 0), (1, 0), 0.25, colors.black),
                    ("INNERGRID", (0, 0), (2, 3), 0.25, colors.black),
                    ("BOX", (0, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "TOP"),
                ]
            )
        )
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            [Paragraph("SECTION THREE", self.styles["section_heading"])],
        ]
        t = Table(data, colWidths=(9 * cm))
        t.hAlign = "LEFT"
        story.append(t)

        data = [
            [
                Paragraph(
                    "Complete this section after group randomization or at "
                    "a time when it is known the subject will NOT advance to "
                    "randomization.",
                    self.styles["section_instructions"],
                )
            ]
        ]
        t = Table(data)
        t.hAlign = "LEFT"
        story.append(t)

        story.append(Spacer(0.1 * cm, 0.5 * cm))

        data = [
            [
                Paragraph(
                    "THE ABOVE DATA HAS BEEN INDEPENDENTLY VERIFIED AGAINST THE EDC",
                    self.styles["line_data_large"],
                )
            ],
        ]
        t = Table(data)
        t.hAlign = "LEFT"
        story.append(t)

        data = [
            (
                [Paragraph("Staff name:", self.styles["line_label"])],
                [Paragraph("Staff signature:", self.styles["line_label"])],
                [Paragraph("Date verified:", self.styles["line_label"])],
            ),
        ]
        t = Table(data, rowHeights=(1 * cm))
        t.setStyle(
            TableStyle(
                [
                    ("BOX", (0, 0), (-1, -1), 0.25, colors.black),
                    ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
                ]
            )
        )
        story.append(t)
        return story

    @property
    def familiar_name(self) -> str | None:
        if self.has_pii_perms():
            return self.object.familiar_name
        return ""

    @property
    def legal_name(self) -> str | None:
        if self.has_pii_perms():
            return self.object.legal_name
        return ""

    def has_pii_perms(self) -> bool:
        has_pii_perms: bool | None = None
        user_sites = set([s.id for s in self.user.userprofile.sites.all()])
        for country in get_remove_patient_names_from_countries():
            country_sites = set([s.site_id for s in get_sites_by_country(country)])
            if user_sites & country_sites:
                has_pii_perms = False
                break
        if has_pii_perms is None and self.user.groups.filter(name__in=[PII, PII_VIEW]):
            has_pii_perms = True
        return has_pii_perms



from __future__ import annotations

import re
from typing import TYPE_CHECKING, Type

from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from edc_constants.constants import NO, TBD, UUID_PATTERN, YES
from edc_dashboard import url_names

from ...models import ConsentRefusal
from ...utils import (
    get_consent_refusal_url,
    get_subject_consent_url,
    get_subject_screening_url,
)

if TYPE_CHECKING:
    from intecomm_consent.models import SubjectConsent, SubjectConsentUg

    from ...models import SubjectScreening, SubjectScreeningUg


class ChangeListTemplateContext:
    """Context for change_list_screen_and_consent.html"""

    subject_dashboard_url_name = "subject_dashboard_url"

    def __init__(
        self,
        patient_log,
        subject_screening_model_cls: Type[SubjectScreening | SubjectScreeningUg] = None,
        subject_consent_model_cls: Type[SubjectConsent | SubjectConsentUg] = None,
    ):
        self._consent_refusal = None
        self._eligible = None
        self._randomized = None
        self._screening_identifier = None
        self._subject_consent = None
        self._subject_identifier = None
        self._subject_screening = None
        self.patient_log = patient_log
        self.subject_screening_model_cls = subject_screening_model_cls
        self.subject_consent_model_cls = subject_consent_model_cls

    @property
    def context(self) -> dict:
        """Context for change_list_screen_and_consent.html"""
        return dict(
            TBD=TBD,
            YES=YES,
            NO=NO,
            consent_refusal=self.consent_refusal,
            consent_refusal_url=self.consent_refusal_url,
            eligible=self.eligible,
            filing_identifier=self.patient_log.filing_identifier,
            group_identifier=self.patient_log.group_identifier,
            patient_log_identifier=self.patient_log.patient_log_identifier,
            randomized=self.randomized,
            screening_identifier=self.screening_identifier,
            stable=self.stable,
            subject_consent=self.subject_consent,
            subject_consent_url=self.subject_consent_url,
            subject_dashboard_url=self.subject_dashboard_url,
            subject_identifier=self.subject_identifier,
            subject_screening_url=self.subject_screening_url,
            willing_to_screen=self.patient_log.willing_to_screen,
        )

    @property
    def screening_identifier(self) -> str | None:
        if not self._screening_identifier:
            if re.match(UUID_PATTERN, self.patient_log.screening_identifier):
                self._screening_identifier = None
            else:
                self._screening_identifier = self.patient_log.screening_identifier
        return self._screening_identifier

    @property
    def subject_screening(self) -> SubjectScreening | SubjectScreeningUg | None:
        if not self._subject_screening:
            try:
                self._subject_screening = self.subject_screening_model_cls.objects.get(
                    screening_identifier=self.screening_identifier
                )
            except ObjectDoesNotExist:
                self._subject_screening = None
        return self._subject_screening

    @property
    def consent_refusal(self) -> ConsentRefusal | None:
        if not self._consent_refusal:
            try:
                self._consent_refusal = ConsentRefusal.objects.get(
                    screening_identifier=self.screening_identifier
                )
            except ObjectDoesNotExist:
                pass
        return self._consent_refusal

    @property
    def subject_identifier(self) -> str | None:
        if not self._subject_identifier:
            if re.match(UUID_PATTERN, self.patient_log.subject_identifier):
                self._subject_identifier = None
            else:
                self._subject_identifier = self.patient_log.subject_identifier
        return self._subject_identifier

    @property
    def subject_consent(self):
        if not self._subject_consent:
            try:
                self._subject_consent = self.subject_consent_model_cls.objects.get(
                    screening_identifier=self.screening_identifier
                )
            except ObjectDoesNotExist:
                pass
        return self._subject_consent

    @property
    def eligible(self) -> bool | None:
        """Returns True if patient is screened eligible."""
        if self._eligible is None:
            try:
                self._eligible = self.subject_screening.eligible
            except AttributeError:
                self._eligible = None
        return self._eligible

    @property
    def randomized(self) -> bool | None:
        """Returns True if patient is screened eligible."""
        if self._randomized is None:
            try:
                self._randomized = True if self.patient_log.group_identifier else False
            except AttributeError:
                self._randomized = None
        return self._randomized

    @property
    def stable(self) -> str:
        return (
            self.patient_log.get_stable_display()
            if self.patient_log.stable != TBD
            else TBD.upper()
        )

    @property
    def subject_screening_url(self) -> str | None:
        if self.patient_log.willing_to_screen != YES:
            url = None
        else:
            url = get_subject_screening_url(
                self.patient_log,
                subject_screening=self.subject_screening,
                subject_screening_model_cls=self.subject_screening_model_cls,
            )
        return url

    @property
    def subject_consent_url(self) -> str | None:
        url = None
        if self.subject_screening and self.subject_screening.eligible:
            url = get_subject_consent_url(
                subject_screening=self.subject_screening,
                subject_consent=self.subject_consent,
                subject_consent_model_cls=self.subject_consent_model_cls,
            )
        return url

    @property
    def consent_refusal_url(self) -> str | None:
        url = None
        if self.consent_refusal or (
            self.subject_screening and self.subject_screening.eligible
        ):
            url = get_consent_refusal_url(
                screening_identifier=self.subject_screening.screening_identifier,
                consent_refusal=self.consent_refusal,
            )
        return url

    @property
    def subject_dashboard_url(self) -> str | None:
        url = None
        if self.randomized:
            url = reverse(
                url_names.get(self.subject_dashboard_url_name),
                kwargs=dict(subject_identifier=self.subject_identifier),
            )
        return url



from __future__ import annotations

from typing import Type

from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from django.utils.html import format_html
from edc_constants.constants import DM, HIV, HTN, NO, TBD, YES
from edc_dashboard import url_names
from edc_form_validators import INVALID_ERROR
from edc_sites.get_country import get_current_country
from intecomm_form_validators import SubjectScreeningFormValidator

from ...constants import UGANDA
from ...models import PatientLog, PatientLogUg


class SubjectScreeningModelFormMixin:
    form_validator_cls = SubjectScreeningFormValidator

    def __init__(self, *args, **kwargs):
        self._patient_log = None
        super().__init__(*args, **kwargs)

    def clean(self) -> dict:
        self.validate_willing_to_screen_on_patient_log()
        self.validate_gender_against_patient_log()
        self.validate_stable_in_care_on_patient_log()
        self.validate_health_talks_on_patient_log()
        self.validate_initials_against_patient_log()
        self.validate_age_in_years_against_patient_log()
        self.validate_hospital_identifier_against_patient_log()
        self.validate_condition(HIV, "hiv_dx")
        self.validate_condition(DM, "dm_dx")
        self.validate_condition(HTN, "htn_dx")

        return super().clean()

    @property
    def patient_log_model_cls(self) -> Type[PatientLog | PatientLogUg]:
        pass

    @property
    def patient_log(self) -> PatientLog | PatientLogUg:
        if not self._patient_log:
            try:
                self._patient_log = self.patient_log_model_cls.objects.get(
                    patient_log_identifier=self.cleaned_data.get("patient_log_identifier")
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    {"patient_log_identifier": "Invalid. Patient log identifier not found."}
                )
            if self._patient_log:
                if get_current_country(site=self._patient_log.site) == UGANDA:
                    self._patient_log = PatientLogUg.objects.get(
                        patient_log_identifier=self.cleaned_data.get("patient_log_identifier")
                    )
        return self._patient_log

    @property
    def patient_log_link(self):
        return (
            f'<a href="{self.patient_log.get_changelist_url()}?'
            f'q={str(self.patient_log.id)}">{self.patient_log._meta.verbose_name}</a>'
        )

    def validate_willing_to_screen_on_patient_log(self):
        if self.patient_log.willing_to_screen:
            if self.patient_log.willing_to_screen == NO:
                errmsg = format_html(
                    f"Invalid. Patient is unwilling to screen. See {self.patient_log_link}."
                )
                raise forms.ValidationError(errmsg, code=INVALID_ERROR)
            elif self.patient_log.willing_to_screen == TBD:
                errmsg = format_html(
                    "Invalid. Patient has not yet agreed to screen. "
                    f"See {self.patient_log_link}."
                )
                raise forms.ValidationError(errmsg, code=INVALID_ERROR)

    def validate_stable_in_care_on_patient_log(self) -> None:
        if self.patient_log.stable != YES:
            errmsg = format_html(
                "Invalid. Patient is NOT known to be stable and in-care. "
                f"See {self.patient_log_link}."
            )
            raise forms.ValidationError(errmsg, code=INVALID_ERROR)

    def validate_health_talks_on_patient_log(self) -> None:
        if self.patient_log.first_health_talk not in [YES, NO]:
            errmsg = format_html(
                "Invalid. Has patient attended the first health talk? "
                f"See {self.patient_log_link}."
            )
            raise forms.ValidationError(errmsg, code=INVALID_ERROR)
        elif self.patient_log.second_health_talk not in [YES, NO]:
            errmsg = format_html(
                "Invalid. Has patient attended the second health talk? "
                f"See {self.patient_log_link}."
            )
            raise forms.ValidationError(errmsg, code=INVALID_ERROR)

    def validate_gender_against_patient_log(self) -> None:
        if self.cleaned_data.get("gender") != self.patient_log.gender:
            raise forms.ValidationError(
                {
                    "gender": format_html(
                        f"Invalid. Expected {self.patient_log.get_gender_display()}. "
                        f"See {self.patient_log_link}."
                    )
                },
                code=INVALID_ERROR,
            )

    def validate_age_in_years_against_patient_log(self):
        if self.cleaned_data.get("age_in_years") != self.patient_log.age_in_years:
            raise forms.ValidationError(
                {
                    "age_in_years": format_html(
                        f"Invalid. Expected {self.patient_log.age_in_years}. "
                        f"See {self.patient_log_link}."
                    )
                },
                code=INVALID_ERROR,
            )

    def validate_initials_against_patient_log(self):
        if self.cleaned_data.get("initials") != self.patient_log.initials:
            raise forms.ValidationError(
                {
                    "initials": format_html(
                        f"Invalid. Expected {self.patient_log.initials}. "
                        f"See {self.patient_log_link}."
                    )
                },
                code=INVALID_ERROR,
            )

    def validate_hospital_identifier_against_patient_log(self):
        if (
            self.cleaned_data.get("hospital_identifier")
            != self.patient_log.hospital_identifier
        ):
            raise forms.ValidationError(
                {
                    "hospital_identifier": format_html(
                        f"Invalid. Expected {self.patient_log.hospital_identifier}. "
                        f"See {self.patient_log_link}."
                    )
                },
                code=INVALID_ERROR,
            )

    def validate_condition(self, name, field):
        if self.patient_log.conditions.count() == 0:
            raise forms.ValidationError(
                format_html(
                    "No conditions (HIV/DM/HTN) have been indicated for this patient. "
                    f"See {self.patient_log_link}."
                ),
                code=INVALID_ERROR,
            )
        else:
            if (
                not self.patient_log.conditions.filter(name=name)
                and self.cleaned_data.get(field) == YES
            ):
                raise forms.ValidationError(
                    {
                        field: format_html(
                            "Invalid. Condition not indicated "
                            f"on the Patient Log. Got {name.upper()}. "
                            f"See {self.patient_log_link}."
                        ),
                    },
                    code=INVALID_ERROR,
                )
            elif (
                self.patient_log.conditions.filter(name=name)
                and self.cleaned_data.get(field) == NO
            ):
                raise forms.ValidationError(
                    {
                        field: format_html(
                            f"Invalid. {name.upper()} was indicated "
                            "as a condition on the Patient Log. "
                            f"See {self.patient_log_link}."
                        ),
                    },
                    code=INVALID_ERROR,
                )

    def already_consented_validation_url(self, cleaned_data: dict | None = None) -> str:
        if self.instance.patient_log.group_identifier:
            url_name = url_names.get("subject_dashboard_url")
            url = reverse(
                url_name,
                kwargs={"subject_identifier": self.instance.subject_identifier},
            )
        else:
            if get_current_country(site=self.instance.site) == UGANDA:
                url = reverse(
                    "intecomm_screening_admin:intecomm_screening_patientlogug_changelist"
                )
            else:
                url = reverse(
                    "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
                )
            url = f"{url}?q={self.instance.subject_identifier}"
        return url



from intecomm_lists.constants import HC_III, HC_IV

from .intecomm_site import IntecommSite

fqdn = "intecomm.clinicedc.org"

ug_language_codes = ["en", "lg", "ry", "sw"]
tz_language_codes = ["en", "sw", "mas"]

all_sites = {
    "uganda": (
        IntecommSite(
            101,
            "kasangati",
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            health_facility_type=HC_IV,
            domain=f"kasangati.ug.{fqdn}",
        ),
        IntecommSite(
            102,
            "kisugu",
            health_facility_type=HC_III,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"kisugu.ug.{fqdn}",
        ),
        IntecommSite(
            103,
            "kiswa",
            health_facility_type=HC_III,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"kiswa.ug.{fqdn}",
        ),
        IntecommSite(
            104,
            "kyazanga",
            health_facility_type=HC_IV,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"kyazanga.ug.{fqdn}",
        ),
        IntecommSite(
            105,
            "mpigi",
            health_facility_type=HC_IV,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"mpigi.ug.{fqdn}",
        ),
        IntecommSite(
            106,
            "namayumba",
            health_facility_type=HC_III,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"namayumba.ug.{fqdn}",
        ),
        IntecommSite(
            107,
            "namulonge",
            health_facility_type=HC_III,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"namulonge.ug.{fqdn}",
        ),
        IntecommSite(
            108,
            "ndejje",
            health_facility_type=HC_IV,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"ndejje.ug.{fqdn}",
        ),
        IntecommSite(
            109,
            "sekiwunga",
            health_facility_type=HC_III,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"sekiwunga.ug.{fqdn}",
        ),
        IntecommSite(
            110,
            "wakiso",
            health_facility_type=HC_IV,
            country_code="ug",
            country="uganda",
            language_codes=ug_language_codes,
            domain=f"wakiso.ug.{fqdn}",
        ),
    ),
    "tanzania": (
        IntecommSite(
            201,
            "amana",
            health_facility_type=HC_IV,
            title="Amana Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"amana.tz.{fqdn}",
        ),
        IntecommSite(
            202,
            "bagamoyo",
            health_facility_type=HC_IV,
            title="Bagamoyo Regional Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"bagamoyo.tz.{fqdn}",
        ),
        IntecommSite(
            203,
            "rugambwa",
            health_facility_type=HC_IV,
            title="Cardinal Rugambwa Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"rugambwa.tz.{fqdn}",
        ),
        IntecommSite(
            204,
            "hindu_mandal",
            health_facility_type=HC_IV,
            title="Hindu Mandal Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"hindu-mandal.tz.{fqdn}",
        ),
        IntecommSite(
            205,
            "kisarawe",
            health_facility_type=HC_IV,
            title="Kisarawe District Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"kisarawe.tz.{fqdn}",
        ),
        IntecommSite(
            206,
            "mbagala",
            health_facility_type=HC_III,
            title="Mbagala Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"mbagala.tz.{fqdn}",
        ),
        IntecommSite(
            207,
            "mnazi_moja",
            health_facility_type=HC_III,
            title="Mnazi Moja Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"mnazi.tz.{fqdn}",
        ),
        IntecommSite(
            208,
            "mwananyamala",
            health_facility_type=HC_IV,
            title="Mwananyamala Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"mwananyamala.tz.{fqdn}",
        ),
        IntecommSite(
            209,
            "sinza",
            health_facility_type=HC_III,
            title="Sinza Hospital",
            country_code="tz",
            country="tanzania",
            language_codes=tz_language_codes,
            domain=f"sinza.tz.{fqdn}",
        ),
        IntecommSite(
            210,
            "temeke",
            health_facility_type=HC_IV,
            title="Temeke Hospital",
            country="tanzania",
            country_code="tz",
            language_codes=tz_language_codes,
            domain=f"temeke.tz.{fqdn}",
        ),
    ),
}



from __future__ import annotations

from typing import Type

from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from edc_sites import get_current_country
from edc_subject_dashboard.views import SubjectDashboardView

from intecomm_consent.models import SubjectConsentUg
from intecomm_group.models import PatientGroup
from intecomm_group.utils import get_group_subject_dashboards_url
from intecomm_screening.constants import UGANDA
from intecomm_screening.models import PatientLog, PatientLogUg

from ....model_wrappers import (
    AppointmentModelWrapper,
    SubjectConsentModelWrapper,
    SubjectConsentUgModelWrapper,
)


class DashboardView(SubjectDashboardView):
    consent_model = "intecomm_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "intecomm_subject.subjectvisit"
    history_button_label = _("Audit")

    appointment_model_wrapper_cls = AppointmentModelWrapper
    consent_model_wrapper_cls = SubjectConsentModelWrapper

    @property
    def patient_log_model_cls(self) -> Type[PatientLog | PatientLogUg]:
        if get_current_country(request=self.request) == UGANDA:
            return PatientLogUg
        return PatientLog

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            group = PatientGroup.objects.get(
                group_identifier=context.get("consent").object.group_identifier
            )
        except ObjectDoesNotExist:
            patient_group_url = None
            group_identifier = None
            group_name = None
        else:
            group_identifier = group.group_identifier
            group_name = group.name
            patient_group_url = reverse(
                "intecomm_group_admin:intecomm_group_patientgroup_changelist"
            )
            patient_group_url = f"{patient_group_url}?q={group.name}"
        context.update(
            subject_listboard_url="screen_group_url",
            group_identifier=group_identifier,
            group_name=group_name,
            patient_group_url=patient_group_url,
            patient_log=self.patient_log,
            patient_log_url=self.patient_log_url,
            group_subject_dashboards_url=get_group_subject_dashboards_url(self.patient_log),
        )
        # replace subject_consent_model_wrapper if UG
        if get_current_country(request=self.request) == UGANDA:
            subject_consent = SubjectConsentUg.objects.get(id=context["consent"].object.id)
            subject_consent_model_wrapper = SubjectConsentUgModelWrapper(subject_consent)
            context.update(
                consent=subject_consent_model_wrapper, consents=[subject_consent_model_wrapper]
            )
        return context

    @property
    def patient_log(self):
        try:
            return self.patient_log_model_cls.objects.get(
                subject_identifier=self.subject_identifier
            )
        except ObjectDoesNotExist:
            return None

    @property
    def patient_log_url(self) -> str:
        if self.patient_log:
            country = get_current_country(site=self.patient_log.site)
            if country == UGANDA:
                patient_log_url = reverse(
                    "intecomm_screening_admin:intecomm_screening_patientlogug_changelist"
                )
            else:
                patient_log_url = reverse(
                    "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
                )
            url = f"{patient_log_url}?q={self.filing_identifier}"
        else:
            url = None
        return url

    @property
    def filing_identifier(self) -> str | None:
        if self.patient_log:
            return self.patient_log.filing_identifier
        return None



from edc_list_data.model_mixins import ListModelMixin


class ArvDrugs(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Arv Drugs"
        verbose_name_plural = "Arv Drugs"


class ArvRegimens(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "ARV Regimens"
        verbose_name_plural = "ARV Regimens"


class ClinicServices(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Clinic Services"
        verbose_name_plural = "Clinic Services"


class ClinicalWithdrawalReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Clinical Withdrawal Reasons"
        verbose_name_plural = "Clinical Withdrawal Reasons"


class Conditions(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Conditions"
        verbose_name_plural = "Conditions"


class DmManagement(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Diabetes Management"
        verbose_name_plural = "Diabetes Management"


class HtnManagement(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "HTN Management"
        verbose_name_plural = "HTN Management"


class DmTreatments(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Diabetes Treatments"
        verbose_name_plural = "Diabetes Treatments"


class DrugDispensaries(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Drug Dispensaries"
        verbose_name_plural = "Drug Dispensaries"


class DrugDispensers(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Drug Dispensers"
        verbose_name_plural = "Drug Dispensers"


class HealthAdvisors(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Advisors"
        verbose_name_plural = "Health Advisors"


class HealthInterventionTypes(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Intervention Types"
        verbose_name_plural = "Health Intervention Types"


class HealthServices(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Services"
        verbose_name_plural = "Health Services"


class HealthTalkTypes(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Talk Type"
        verbose_name_plural = "Health Talk Types"


class HealthTalkConditions(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Talk Conditions"
        verbose_name_plural = "Health Talk Conditions"


class HtnTreatments(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Hypertension Treatments"
        verbose_name_plural = "Hypertension Treatments"


class LaboratoryTests(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Laboratory Tests"
        verbose_name_plural = "Laboratory Tests"


class OffstudyReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Offstudy Reasons"
        verbose_name_plural = "Offstudy Reasons"


class MedicationShortageReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Medication shortage reasons"
        verbose_name_plural = "Medication shortage reasons"


class NonAdherenceReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "NonAdherence Reasons"
        verbose_name_plural = "NonAdherence Reasons"


class RefillConditions(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Refill Conditions"
        verbose_name_plural = "Refill Conditions"


class ToxicityWithdrawalReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Toxicity Withdrawal Reasons"
        verbose_name_plural = "Toxicity Withdrawal Reasons"


class VisitReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Visit Reasons"
        verbose_name_plural = "Visit Reasons"


class SubjectVisitMissedReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Subject Missed Visit Reasons"
        verbose_name_plural = "Subject Missed Visit Reasons"


class DrugPaySources(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Drug Payment Sources"
        verbose_name_plural = "Drug Payment Sources"


class TransferReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Transfer Reasons"
        verbose_name_plural = "Transfer Reasons"


class TransportChoices(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Transport Choices"
        verbose_name_plural = "Transport Choices"


class HealthFacilityTypes(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Health Facility Type"
        verbose_name_plural = "Health Facility Types"


class LocationTypes(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Community-care location type"
        verbose_name_plural = "Community-care location types"


class RxModificationReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Treatment Modification Reasons"
        verbose_name_plural = "Treatment Modification Reasons"


class RxModifications(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Treatment Modifications"
        verbose_name_plural = "Treatment Modifications"


class ScreeningRefusalReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Screening Refusal Reasons"
        verbose_name_plural = "Screening Refusal Reasons"


class ConsentRefusalReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Consent Refusal Reasons"
        verbose_name_plural = "Consent Refusal Reasons"



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from ..admin_site import intecomm_subject_admin
from ..models import ArvRegimens


@admin.register(ArvRegimens, site=intecomm_subject_admin)
class ArvRegimensAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



# Generated by Django 4.2.3 on 2023-07-25 21:10
from django.db import migrations
from tqdm import tqdm


def update_next_appointment(apps, schema_editor):
    """Update healthfacility FK from old healthfacility model to
    new healthfacility model.
    """
    model_cls = apps.get_model("intecomm_subject", "nextappointment")
    healthfacility_model_cls = apps.get_model("intecomm_facility", "healthfacility")
    healthfacility_old_model_cls = apps.get_model("intecomm_screening", "healthfacility")
    total = model_cls.objects.all().count()
    print("\n")
    for obj in tqdm(model_cls.objects.all(), total=total):
        if not obj.old_health_facility:
            continue
        old_health_facility = healthfacility_old_model_cls.objects.get(
            id=obj.old_health_facility
        )
        obj.health_facility_id = healthfacility_model_cls.objects.get(
            name=old_health_facility.name
        ).id
        obj.save_base(update_fields=["health_facility_id"])
    print("\n")


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0066_historicalnextappointment_health_facility_and_more"),
    ]

    operations = [migrations.RunPython(update_next_appointment)]



from .signals import (
    subjectconsent_on_pre_delete,
    subjectconsentug_on_pre_delete,
    update_subjectconsent_on_post_save,
    update_subjectconsentug_on_post_save,
)
from .subject_consent import SubjectConsent
from .subject_consent_ug import SubjectConsentUg
from .subject_reconsent import SubjectReconsent



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_screening_admin
from ..models import PatientLogReportPrintHistory
from .modeladmin_mixins import BaseModelAdminMixin


@admin.register(PatientLogReportPrintHistory, site=intecomm_screening_admin)
class PatientLogReportPrintHistoryAdmin(SiteModelAdminMixin, BaseModelAdminMixin):
    show_object_tools = False

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "patient_log_identifier",
                    "printed_datetime",
                    "printed_user",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = ("patient_log_identifier", "printed_datetime", "printed_user")
    list_filter = ("printed_datetime", "printed_user")
    search_fields = ("patient_log_identifier",)
    readonly_fields = ("patient_log_identifier", "printed_datetime", "printed_user")



# Generated by Django 4.1.7 on 2023-04-19 19:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0005_htnmanagement_and_more"),
        ("intecomm_subject", "0032_historicalnextappointment_best_visit_code_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="dm_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="hiv_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="htn_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="dm_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="hiv_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="htn_dx_at_screening",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was this diagnosis reported at screening",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="best_visit_code",
            field=models.CharField(
                choices=[
                    ("1000", "1000"),
                    ("1010", "1010"),
                    ("1020", "1020"),
                    ("1030", "1030"),
                    ("1040", "1040"),
                    ("1050", "1050"),
                    ("1060", "1060"),
                    ("1070", "1070"),
                    ("1080", "1080"),
                    ("1090", "1090"),
                    ("1100", "1100"),
                    ("1110", "1110"),
                    ("1120", "1120"),
                ],
                help_text="Click SAVE to let the EDC suggest. Once selected, interim appointments will be flagged as not required.",
                max_length=15,
                null=True,
                verbose_name="Which study visit code is closest to this appointment date",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="best_visit_code",
            field=models.CharField(
                choices=[
                    ("1000", "1000"),
                    ("1010", "1010"),
                    ("1020", "1020"),
                    ("1030", "1030"),
                    ("1040", "1040"),
                    ("1050", "1050"),
                    ("1060", "1060"),
                    ("1070", "1070"),
                    ("1080", "1080"),
                    ("1090", "1090"),
                    ("1100", "1100"),
                    ("1110", "1110"),
                    ("1120", "1120"),
                ],
                help_text="Click SAVE to let the EDC suggest. Once selected, interim appointments will be flagged as not required.",
                max_length=15,
                null=True,
                verbose_name="Which study visit code is closest to this appointment date",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="missed_reasons",
            field=models.ManyToManyField(
                blank=True, to="intecomm_lists.subjectvisitmissedreasons"
            ),
        ),
    ]



from django import forms
from django.contrib.sites.models import Site
from edc_consent.utils import get_consent_model_cls
from edc_form_validators import FormValidatorMixin
from edc_screening.utils import (
    get_subject_screening_model_cls,
    get_subject_screening_or_raise,
    is_eligible_or_raise,
)
from edc_sites.get_country import get_current_country
from edc_sites.modelform_mixins import SiteModelFormMixin
from intecomm_form_validators import ConsentRefusalFormValidator

from intecomm_consent.utils import raise_if_subject_consent_exists

from ..constants import UGANDA
from ..models import ConsentRefusal


class ConsentRefusalForm(SiteModelFormMixin, FormValidatorMixin, forms.ModelForm):
    form_validator_cls = ConsentRefusalFormValidator

    def clean(self):
        cleaned_data = super().clean()
        screening_identifier = self.cleaned_data.get("screening_identifier")
        if subject_screening := get_subject_screening_or_raise(
            screening_identifier, is_modelform=True
        ):
            is_eligible_or_raise(
                screening_identifier=screening_identifier,
                url_name=self.changelist_url,
            )
            raise_if_subject_consent_exists(
                screening_identifier=screening_identifier,
                subject_screening=subject_screening,
                subject_screening_model_cls=get_subject_screening_model_cls(),
                subject_consent_model_cls=get_consent_model_cls(),
                is_modelform=True,
            )
        return cleaned_data

    @property
    def changelist_url(self):
        if get_current_country(site=Site.objects.get_current()) == UGANDA:
            return "intecomm_screening_admin:intecomm_screening_patientlogug_changlist"
        return "intecomm_screening_admin:intecomm_screening_patientlog_changlist"

    class Meta:
        model = ConsentRefusal
        fields = "__all__"
        help_texts = {"screening_identifier": "(read-only)"}
        widgets = {"screening_identifier": forms.TextInput(attrs={"readonly": "readonly"})}



# Generated by Django 4.2.1 on 2023-07-05 02:29
from django.db import migrations
from django.db.migrations import RunPython
from tqdm import tqdm


def update_patient_log_identifier(apps, schema_editor):
    subjectscreening_cls = apps.get_model("intecomm_screening.subjectscreening")
    historical_cls = apps.get_model("intecomm_screening.historicalsubjectscreening")
    qs = subjectscreening_cls.objects.all()
    for subjectscreening in tqdm(qs, total=qs.count()):
        for obj in historical_cls.objects.filter(
            screening_identifier=subjectscreening.screening_identifier
        ):
            obj.patient_log_identifier = subjectscreening.patient_log.patient_log_identifier
            obj.save_base(update_fields="patient_log_identifier")


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0042_alter_healthfacility_options_and_more"),
    ]

    operations = [RunPython(update_patient_log_identifier)]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from intecomm_form_validators.subject import HivInitialReviewFormValidator

from ..models import HivInitialReview


class HivInitialReviewForm(
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HivInitialReviewFormValidator

    class Meta:
        model = HivInitialReview
        fields = "__all__"



from intecomm_rando.constants import COMMUNITY_ARM

from .base_subject_listboard_view import BaseSubjectListboardView


class CommunitySubjectListboardView(BaseSubjectListboardView):
    listboard_template = "community_subject_listboard_template"
    listboard_url = "community_subject_listboard_url"
    navbar_selected_item = "comm_followup"
    search_form_url = "community_subject_listboard_url"
    listboard_fa_icon = "fas fa-users-between-lines fa-2x"
    assignment = COMMUNITY_ARM
    followup_url = "followup_community_url"
    listboard_panel_title = "Community group participants"



from django.contrib import admin
from edc_he.modeladmin_mixins import HealthEconomicsIncomeModelAdminMixin

from ...admin_site import intecomm_subject_admin
from ...forms import HealthEconomicsIncomeForm
from ...models import HealthEconomicsIncome
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(HealthEconomicsIncome, site=intecomm_subject_admin)
class HealthEconomicsIncomeAdmin(HealthEconomicsIncomeModelAdminMixin, CrfModelAdmin):
    form = HealthEconomicsIncomeForm
